@@ -1,4 +1,5 @@
 from functools import partial
+import time
 from collections import defaultdict
 from random import random
 
@@ -10,7 +11,6 @@ from .hv_battle_item_provider import ItemProvider
 from .hv_battle_action_manager import ElementActionManager
 from .hv_battle_skill_manager import SkillManager
 from .hv_battle_buff_manager import BuffManager
-from .hv_battle_monster_status_manager import MonsterStatusManager
 
 # from .hv_battle_dashboard import BattleDashBoard
 from .pause_controller import PauseController
@@ -71,7 +71,6 @@ class BattleDriver(HVDriver):
         self._itemprovider = ItemProvider(self, self.battle_dashboard)
         self._skillmanager = SkillManager(self, self.battle_dashboard)
         self._buffmanager = BuffManager(self, self.battle_dashboard)
-        self._monsterstatusmanager = MonsterStatusManager(self, self.battle_dashboard)
         self.pausecontroller = PauseController()
         self.turn = -1
         self.round = -1
@@ -79,7 +78,6 @@ class BattleDriver(HVDriver):
 
     def clear_cache(self) -> None:
         # 重新解析戰鬥儀表板以獲取最新的怪物狀態
-        self._monsterstatusmanager.clear_cache()
         self.round = self.battle_dashboard.log_entries.current_round
         self.battle_dashboard.update()
 
@@ -95,18 +93,19 @@ class BattleDriver(HVDriver):
     def click_skill(self, key: str, iswait=True) -> bool:
         if key in self.forbidden_skills:
             return False
-        return self._skillmanager.cast(key, iswait=iswait)
+        result = self._skillmanager.cast(key, iswait=iswait)
+        return result
 
     def get_stat_percent(self, stat: str) -> float:
         match stat.lower():
             case "hp":
-                value = self.battle_dashboard.character_vitals.hp
+                value = self.battle_dashboard.snap.player.hp_percent
             case "mp":
-                value = self.battle_dashboard.character_vitals.mp
+                value = self.battle_dashboard.snap.player.mp_percent
             case "sp":
-                value = self.battle_dashboard.character_vitals.sp
+                value = self.battle_dashboard.snap.player.sp_percent
             case "overcharge":
-                value = self.battle_dashboard.character_vitals.overcharge
+                value = self.battle_dashboard.snap.player.overcharge_value
             case _:
                 raise ValueError(f"Unknown stat: {stat}")
         return value
@@ -123,6 +122,8 @@ class BattleDriver(HVDriver):
         return self._itemprovider.use(key)
 
     def apply_buff(self, key: str, force: bool = False) -> bool:
+        if key in self.forbidden_skills:
+            return False
         apply_buff = partial(self._buffmanager.apply_buff, key=key, force=force)
         if not force:
             match key:
@@ -143,10 +144,6 @@ class BattleDriver(HVDriver):
                         return False
         return apply_buff()
 
-    @property
-    def system_monster_alive_ids(self) -> list[int]:
-        return self._monsterstatusmanager.alive_system_monster_ids
-
     def check_hp(self) -> bool:
         if self.get_stat_percent("hp") < self.statthreshold.hp[0]:
             for fun in [
@@ -236,14 +233,12 @@ class BattleDriver(HVDriver):
         return False
 
     def go_next_floor(self) -> bool:
-        continue_elements = self.driver.find_elements(By.ID, "btcp")
-
-        if continue_elements:
-            self.element_action_manager.click_and_wait_log(continue_elements[0])
+        # Use locator-based click to avoid stale element caching
+        if self.driver.find_elements(By.ID, "btcp"):
+            self.element_action_manager.click_and_wait_log_locator(By.ID, "btcp")
             self._create_last_debuff_monster_id()
             return True
-        else:
-            return False
+        return False
 
     def debuff_monster(self, debuff: str, nums: list[int]) -> bool:
         debuff_skill = MONSTER_DEBUFF_TO_CHARACTER_SKILL[debuff]
@@ -251,7 +246,9 @@ class BattleDriver(HVDriver):
             return False
 
         monster_ids_with_debuff = (
-            self._monsterstatusmanager.get_monster_ids_with_debuff(debuff)
+            self.battle_dashboard.overview_monsters.alive_monster_with_buff.get(
+                debuff, []
+            )
         ) + [self.last_debuff_monster_id[debuff]]
         for num in nums:
             if num not in monster_ids_with_debuff:
@@ -263,17 +260,15 @@ class BattleDriver(HVDriver):
         return False
 
     def attack_monster(self, n: int) -> bool:
-        elements = self.driver.find_elements(By.ID, "mkey_{n}".format(n=n))
-
-        if not elements:
+        element_id = f"mkey_{n}"
+        if not self.driver.find_elements(By.ID, element_id):
             return False
-
-        self.element_action_manager.click_and_wait_log(elements[0])
+        self.element_action_manager.click_and_wait_log_locator(By.ID, element_id)
         return True
 
-    def attack_monster_by_skill(self, n: int, skill_name: str):
+    def attack_monster_by_skill(self, n: int, skill_name: str) -> bool:
         self.click_skill(skill_name, iswait=False)
-        self.attack_monster(n)
+        return self.attack_monster(n)
 
     def attack(self) -> bool:
         base_monster_ids = [1, 3, 5, 7, 9, 2, 4, 6, 8, 0]
@@ -287,16 +282,22 @@ class BattleDriver(HVDriver):
             monster_alive_ids = [
                 id
                 for id in bmlist
-                if id in self._monsterstatusmanager.alive_monster_ids
+                if id in self.battle_dashboard.overview_monsters.alive_monster
             ]
-            if len(self.system_monster_alive_ids):
+            if len(self.battle_dashboard.overview_monsters.alive_monster):
                 monster_alive_ids = monster_ids_starting_with(
-                    monster_alive_ids, self.system_monster_alive_ids[0]
+                    monster_alive_ids,
+                    self.battle_dashboard.overview_monsters.alive_monster[0],
                 )
             for monster_name in ["Yggdrasil", "Skuld", "Urd", "Verdandi"][::-1]:
-                monster_id = self._monsterstatusmanager.get_monster_id_by_name(
+                if (
                     monster_name
-                )
+                    not in self.battle_dashboard.overview_monsters.alive_monster_name
+                ):
+                    continue
+                monster_id = self.battle_dashboard.overview_monsters.alive_monster_name[
+                    monster_name
+                ]
                 if monster_id in monster_alive_ids:
                     monster_alive_ids = monster_ids_starting_with(
                         monster_alive_ids, monster_id
@@ -308,13 +309,16 @@ class BattleDriver(HVDriver):
             self.with_ofc
             and self.get_stat_percent("overcharge") > 220
             and self._buffmanager.has_buff("Spirit Stance")
-            and self._monsterstatusmanager.alive_count
+            and len(self.battle_dashboard.overview_monsters.alive_monster)
             >= self.statthreshold.countmonster[1]
-            and self._skillmanager.get_skill_status("Orbital Friendship Cannon")
-            == "available"
+            and "Orbital Friendship Cannon"
+            in self.battle_dashboard.snap.abilities.skills
+            and self.battle_dashboard.snap.abilities.skills[
+                "Orbital Friendship Cannon"
+            ].available
         ):
             self.attack_monster_by_skill(
-                self._monsterstatusmanager.alive_monster_ids[0],
+                self.battle_dashboard.overview_monsters.alive_monster[0],
                 "Orbital Friendship Cannon",
             )
             return True
@@ -330,13 +334,12 @@ class BattleDriver(HVDriver):
             for debuff in MONSTER_DEBUFF_TO_CHARACTER_SKILL:
                 if debuff in ["Imperiled"]:
                     continue
-                if (
-                    len(monster_alive_ids)
-                    - len(
-                        self._monsterstatusmanager.get_monster_ids_with_debuff(debuff)
+                debuffed_monsters = (
+                    self.battle_dashboard.overview_monsters.alive_monster_with_buff.get(
+                        debuff, []
                     )
-                    < 3
-                ):
+                )
+                if len(monster_alive_ids) - len(debuffed_monsters) < 3:
                     continue
                 if self.debuff_monster(debuff, monster_alive_ids):
                     return True
@@ -347,50 +350,68 @@ class BattleDriver(HVDriver):
             and self.get_stat_percent("mp") > self.statthreshold.mp[1]
         ):
             monster_with_imperil = (
-                self._monsterstatusmanager.get_monster_ids_with_debuff("Imperiled")
+                self.battle_dashboard.overview_monsters.alive_monster_with_buff.get(
+                    "Imperiled", []
+                )
             )
         else:
             monster_with_imperil = monster_alive_ids
 
-        n = monster_alive_ids[0]
-        if n in monster_with_imperil:
-            if self.get_stat_percent("overcharge") > 200 and self._buffmanager.has_buff(
-                "Spirit Stance"
-            ):
-                monster_health = self.battle_dashboard.monster_list.monsters[
-                    n
-                ].vitals.health
-                if monster_health < 25 and self._skillmanager.get_skill_status(
-                    "Merciful Blow"
-                ):
-                    self.attack_monster_by_skill(n, "Merciful Blow")
-                elif self._skillmanager.get_skill_status("Vital Strike"):
-                    self.attack_monster_by_skill(n, "Vital Strike")
-            else:
-                self.attack_monster(n)
-            self.last_debuff_monster_id["Imperiled"] = -1
-        else:
-            if n == self.last_debuff_monster_id["Imperiled"]:
-                # If the last debuffed monster is the same, attack it directly
-                if random() < 0.5:
-                    self.attack_monster_by_skill(n, "Imperil")
+        if monster_alive_ids:
+            n = monster_alive_ids[0]
+            if n in monster_with_imperil:
+                if self.get_stat_percent(
+                    "overcharge"
+                ) > 200 and self._buffmanager.has_buff("Spirit Stance"):
+                    monster_health = self.battle_dashboard.snap.monsters[n].hp_percent
+                    if (
+                        monster_health < 25
+                        and "Merciful Blow"
+                        in self.battle_dashboard.snap.abilities.skills
+                        and self.battle_dashboard.snap.abilities.skills[
+                            "Merciful Blow"
+                        ].available
+                    ):
+                        self.attack_monster_by_skill(n, "Merciful Blow")
+                    elif (
+                        monster_health > 5
+                        and "Vital Strike"
+                        in self.battle_dashboard.snap.abilities.skills
+                        and self.battle_dashboard.snap.abilities.skills[
+                            "Vital Strike"
+                        ].available
+                    ):
+                        self.attack_monster_by_skill(n, "Vital Strike")
+                    else:
+                        self.attack_monster(n)
                 else:
                     self.attack_monster(n)
+                self.last_debuff_monster_id["Imperiled"] = -1
             else:
-                self.attack_monster_by_skill(
-                    n, MONSTER_DEBUFF_TO_CHARACTER_SKILL["Imperiled"]
-                )
-                self.last_debuff_monster_id["Imperiled"] = n
-        return True
+                if n == self.last_debuff_monster_id["Imperiled"]:
+                    if random() < 0.5:
+                        self.attack_monster_by_skill(n, "Imperil")
+                    else:
+                        self.attack_monster(n)
+                else:
+                    self.attack_monster_by_skill(
+                        n, MONSTER_DEBUFF_TO_CHARACTER_SKILL["Imperiled"]
+                    )
+                    self.last_debuff_monster_id["Imperiled"] = n
+            return True
+        else:
+            return False
 
     def use_channeling(self) -> bool:
-        if "Channeling" in self.battle_dashboard.character_buffs.buffs:
+        if "Channeling" in self.battle_dashboard.snap.player.buffs:
             skill_names = ["Regen", "Heartseeker"]
             skill2remaining: dict[str, float] = dict()
             for skill_name in skill_names:
                 remaining_turns = self._buffmanager.get_buff_remaining_turns(skill_name)
                 refresh_turns = self._buffmanager.skill2turn[skill_name]
-                skill_cost = self._skillmanager.get_skill_mp_cost_by_name(skill_name)
+                skill_cost = self._skillmanager.get_max_skill_mp_cost_by_name(
+                    skill_name
+                )
                 skill2remaining[skill_name] = (
                     (refresh_turns - remaining_turns) * refresh_turns / skill_cost
                 )
@@ -401,11 +422,12 @@ class BattleDriver(HVDriver):
 
         return False
 
-    def battle_in_turn(self) -> str:
+    def battle_in_turn(self) -> bool:
         self.turn += 1
         self.clear_cache()
         # Print the current round logs
-        print("\n".join(self.new_logs))
+        if self.new_logs:
+            print("\n".join(self.new_logs))
 
         for fun in [
             self.go_next_floor,
@@ -419,38 +441,34 @@ class BattleDriver(HVDriver):
             lambda: self.apply_buff("Spirit Draught"),
             lambda: self.apply_buff("Regen"),
             lambda: self.apply_buff("Scroll of Life"),
+            lambda: self.apply_buff("Scroll of Absorption"),
             lambda: self.apply_buff("Absorb"),
             lambda: self.apply_buff("Scroll of Protection"),
             lambda: self.apply_buff("Heartseeker"),
+            self.use_channeling,
+            self.attack,
         ]:
             if fun():
-                return "continue"
+                return True
 
-        if self.use_channeling():
-            return "continue"
-
-        if self.attack():
-            return "continue"
-
-        return "break"
+        return False
 
     def _create_last_debuff_monster_id(self) -> None:
         self.last_debuff_monster_id: dict[str, int] = defaultdict(lambda: -1)
 
     def battle(self) -> None:
         self._create_last_debuff_monster_id()
-        partial_click_skill_menu = partial(
-            self.element_action_manager.click,
-            self.driver.find_element(By.ID, "ckey_skill"),
-        )
-        partial_click_skill_menu()
-        partial_click_skill_menu()
+        # Open skill menu twice using resilient locator-based click (no element caching & no log wait)
+        from selenium.webdriver.common.by import (
+            By as _By,
+        )  # local import to avoid top-level clutter
+
+        for _ in range(2):
+            self.element_action_manager.click_resilient(
+                lambda: self.driver.find_element(_By.ID, "ckey_skill")
+            )
+            time.sleep(0.05)
 
         while True:
-            match self.pausecontroller.pauseable(self.battle_in_turn)():
-                case "break":
-                    break
-                case "continue":
-                    continue
-                case _:
-                    raise ValueError("Unexpected return value from battle_in_turn.")
+            if not self.pausecontroller.pauseable(self.battle_in_turn)():
+                break
