@@ -1,5 +1,3 @@
-from collections import defaultdict
-
 from selenium.webdriver.common.by import By
 from selenium.webdriver.remote.webdriver import WebDriver, WebElement
 
@@ -13,10 +11,10 @@ GEM_ITEMS = {"Mystic Gem", "Health Gem", "Mana Gem", "Spirit Gem"}
 class ItemProvider:
     def __init__(self, driver: HVDriver, battle_dashboard: BattleDashboard) -> None:
         self.hvdriver: HVDriver = driver
+        self.battle_dashboard = battle_dashboard
         self.element_action_manager = ElementActionManager(
             self.hvdriver, battle_dashboard
         )
-        self._checked_items: dict[str, str] = defaultdict(lambda: "available")
 
     @property
     def driver(self) -> WebDriver:
@@ -27,7 +25,10 @@ class ItemProvider:
         return self.hvdriver.driver.find_element(By.ID, "ckey_items")
 
     def click_items_menu(self) -> None:
-        self.element_action_manager.click(self.items_menu_web_element)
+        # Resilient click to mitigate stale menu button
+        self.element_action_manager.click_resilient(
+            lambda: self.hvdriver.driver.find_element(By.ID, "ckey_items")
+        )
 
     def is_open_items_menu(self) -> bool:
         """
@@ -39,27 +40,6 @@ class ItemProvider:
     def get_pane_items(self) -> WebElement:
         return self.hvdriver.driver.find_element(By.ID, "pane_item")
 
-    def get_item_status(self, item: str) -> str:
-        """
-        回傳 'available', 'unavailable', 'not_found'
-        """
-        if self._checked_items[item] == "not_found":
-            return "not_found"
-
-        item_divs = self.get_pane_items().find_elements(
-            By.XPATH, f"//div/div[text()='{item}']"
-        )
-        if not item_divs:
-            if item not in GEM_ITEMS:
-                self._checked_items[item] = "not_found"
-            return "not_found"
-
-        for div in item_divs:
-            parent = div.find_element(By.XPATH, "./ancestor::div[2]")
-            if parent.get_attribute("id") and parent.get_attribute("onclick"):
-                return "available"
-        return "unavailable"
-
     def get_item_elements(self, item: str) -> list[WebElement]:
         return self.get_pane_items().find_elements(
             By.XPATH,
@@ -69,10 +49,10 @@ class ItemProvider:
         )
 
     def use(self, item: str) -> bool:
-        if self._checked_items[item] == "not_found":
+        if item not in self.battle_dashboard.snap.items.items:
             return False
 
-        if self.get_item_status(item) == "unavailable":
+        if not self.battle_dashboard.snap.items.items[item].available:
             return False
 
         item_button_list = self.get_item_elements(item)
@@ -82,5 +62,11 @@ class ItemProvider:
         if not self.is_open_items_menu():
             self.click_items_menu()
             item_button_list = self.get_item_elements(item)
-        self.element_action_manager.click_and_wait_log(item_button_list[0])
+        # Use locator-based click (derive unique locator via first element's id attribute if present)
+        target = item_button_list[0]
+        item_id = target.get_attribute("id")
+        if item_id:
+            self.element_action_manager.click_and_wait_log_locator(By.ID, item_id)
+        else:
+            raise ValueError("Item ID not found")
         return True
